package com.fundi.framework.common.comparator;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.util.ArrayList;

import com.fundi.framework.common.displaylist.DisplayList;
import com.fundi.framework.common.displaylist.DisplayListDeserializer;
import com.fundi.framework.common.editor.SavedDisplay;
import com.fundi.framework.common.instance.AppInstance;
import com.fundi.framework.common.navigation.INavigationType;
import com.fundi.framework.common.nl1.Messages;
import com.fundi.framework.common.util.Collator;

public class ComparatorSource implements INavigationType, Comparable, Serializable {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private static final long serialVersionUID = 1L;
	
	private int id = 0;  //This value is generated by the AppInstance when it is "registered".
	private String title = ""; //$NON-NLS-1$
	private Object data = null;  //A DisplayList or a File containing a SavedDisplayComparatorSource.
	private ArrayList<ComparatorSource> instantSources = new ArrayList<ComparatorSource>();  //Extra sources for comparing without the configuration dialog.
	private boolean instantComparison = false;
	
	public boolean isInstantComparison() {
		return instantComparison;
	}

	public void setInstantComparison(boolean instantComparison) {
		this.instantComparison = instantComparison;
	}

	public ComparatorSource(String title, Object data) {

		super();
		
		this.title = title;
		this.data = data;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ComparatorSource other = (ComparatorSource) obj;
		if (id != other.id)
			return false;
		return true;
	}

	@Override
	public String getNavigationTypeName() {
		
		return null;
	}

	@Override
	public String getDisplayName() {
		
		return Messages.getString("ComparatorSource_1"); //$NON-NLS-1$
	}

	@Override
	public String getFullTitle() {
		
		return Messages.getString("ComparatorSource_2"); //$NON-NLS-1$
	}

	@Override
	public int getConnectStatus() {

		return 0;
	}

	@Override
	public void setConnectStatus(int status) {
		
	}

	public Object getData(AppInstance instance) {
		
		ObjectInputStream in;
		Object obj;
		Object titles;
		DisplayList list = null;
		ArrayList<String> titlesList;
		ArrayList<DisplayList> lists;
		int idx;
		ArrayList<DisplayListDeserializer> deserializers = null;
		DisplayListDeserializer deserializer = null;
		
		if (SavedDisplay.class.isInstance(data)) {
			data = instance.getSavedDisplayManager().getListFromSaved((SavedDisplay) data, this.getTitle());
			return data;
		}
		else {
			
			/*
			 *  If the data is a File, then it contains a SavedDisplayComparatorSource from which we need to retrieve a DisplayList.
			 *
			 *   It is possible for a saved display comparator source to contain multiple lists, in which case it also has a corresponding array
			 *   of multiple titles (CEX plug-in does this for its session and status monitor editors).  If the titles is a list, then the list is
			 *   also, so retrieve the list with the corresponding title.
			 */
			if (File.class.isInstance(data)) {  //File of an old-style saved display.
				try {
					deserializers = instance.getDisplayListDeserializers();
				
					in = new ObjectInputStream(new FileInputStream((File) data));
					obj = in.readObject();
					//If it's a string and the value starts with "deserializer=", we need to use the deserializer extension to deserialize the display list(s). 
					if (String.class.isInstance(obj)) {
						if (((String) obj).toLowerCase().startsWith(Messages.getString("ComparatorSource_0"))) { //$NON-NLS-1$
							for (idx = 0; idx < deserializers.size(); idx++) {
								if (deserializers.get(idx).getClass().getName().equals(((String) obj).substring(13))) {
									deserializer = deserializers.get(idx);
								}
							}
						}
					}
				
					if (deserializer != null) {
						titles = in.readObject();
					}
					else {
						titles = obj;  //There was no deserializer specified, so already got the title/titles.
					}
					if (ArrayList.class.isInstance(titles)) {
						titlesList = (ArrayList<String>) titles;
						if (deserializer == null) {
							lists = (ArrayList<DisplayList>) in.readObject();
						}
						else {
							lists = deserializer.getLists(in);
						}
						for (idx = 0; idx < titlesList.size(); idx++) {
							if (titlesList.get(idx).equals(getTitle())) {
								list = lists.get(idx);
								break;
							}
						}
					}
					else {
						title = (String) titles;
						if (deserializer == null) {
							list = (DisplayList) in.readObject();
						}
						else {
							list = deserializer.getList(in);
						}
					}
					in.close();
					data = list;
					return data;
				}
				catch (Throwable e) {}
			}
			else {
				return data;
			}
		}
		
		return null;
	}

	public void setData(Object data) {
		this.data = data;
	}
	
	@Override
	public String toString() {
		
		return getTitle();
	}

	@Override
	public int compareTo(Object arg0) {

		return (new Collator()).compare(toString().toLowerCase(), arg0.toString().toLowerCase());
	}
	
	public void addInstantSource(ComparatorSource source) {
		
		instantSources.add(source);
	}
	
	public ArrayList<ComparatorSource> getInstantSources() {
		
		return instantSources;
	}

	@Override
	public int testConnection(AppInstance instance) {

		return AppInstance.CAN_CONNECT_UNKNOWN;
	}
}
