package com.fundi.framework.common.displaylist;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.io.Serializable;

import javax.xml.bind.annotation.XmlRootElement;

import com.fundi.framework.common.displaylist.PropDefn.EditResult;

@XmlRootElement
public class ListField implements Serializable {
	
	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private static final long serialVersionUID = 1L;
	
	private ListFieldDefn defn = null;
	private String value = ""; //$NON-NLS-1$
	private boolean hasDifferentValue = false;  //This only has meaning in a comparator scenario, as it's comparing to the previous row's value.
	private ListFieldColor color = new ListFieldColor();

	public ListField() {
	
		super();
	}
	
	public ListField(ListFieldDefn defn, String value) {

		super();
		
		this.defn = defn;
		this.value = value;
	}
	
	public ListField (ListFieldDefn defn, ListField sourceField) {

		//This constructor is designed for Comparator use.
		this.defn = new ListFieldDefn(defn);
		value = sourceField.getValue();
	}

	public ListFieldDefn getDefn() {
		
		return defn;
	}

	public void setDefn(ListFieldDefn defn) {
		
		this.defn = defn;
	}

	public String getValue() {
		
		if (value == null) {
			return ""; //$NON-NLS-1$
		}
		else {
			return value;
		}
	}
	
	public int getIntValue() {
		
		return Integer.parseInt(value);
	}
	
	public void setValue(String value) {
		
		this.value = value;
	}
	
	public String getLabel() {
		
		return defn.getLabel();
	}
	
	public String getCompareLabel() {
		
		return defn.getCompareLabel();
	}
	
	public String getDataType() {
		
		return defn.getDataType();
	}
	
	public String getAlignment() {
		
		return defn.getAlignment();
	}
	
	public boolean isKey() {
		
		return defn.isKey();
	}
	
	public boolean isVisible() {
		
		return defn.isVisible();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((defn == null) ? 0 : defn.hashCode());
		result = prime * result + ((value == null) ? 0 : value.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ListField other = (ListField) obj;
		if (defn == null) {
			if (other.defn != null)
				return false;
		} else if (!defn.equals(other.defn))
			return false;
		if (value == null) {
			if (other.value != null)
				return false;
		} else if (!value.equals(other.value))
			return false;
		return true;
	}

	public boolean hasDifferentValue() {
		
		return hasDifferentValue;
	}

	public void setHasDifferentValue(boolean hasDifferentValue) {
		
		this.hasDifferentValue = hasDifferentValue;
	}

	public void resetColor() {
		
		if (color == null) {
			color = new ListFieldColor();
		}
		else {
			color.reset();
		}
	}
	
	public EditResult editValue() {
		
		if (getPropDefn() != null) {
			return getPropDefn().edit(value);
		}
		else {
			return PropDefn.EditResult.OK;
		}
	}
	
	public String getBooleanRepresentation(boolean val) {
		
		//Return the String representation of the boolean value as according to the property definition.
		if (getPropDefn() != null) {
			return getPropDefn().getBooleanRepresentation(val);
		}
		else {
			return null;
		}
	}

	public void setBooleanRepresentation(boolean val) {
		
		value = getBooleanRepresentation(val);
	}
	
	public String getDefault() {
		
		if (getPropDefn() != null) {
			return getPropDefn().getDefaultValue();
		}
		return null;
	}
	
	public void setDefault() {
		
		value = getDefault();
	}
	
	public PropDefn getPropDefn() {
		
		return defn.getPropDefn();
	}

	@Override
	public String toString() {
		StringBuilder	sb = new StringBuilder();
		String			alias;
		
		sb.append(getLabel());
		alias = getDefn().getAlias();
		if (alias != null && !alias.isEmpty()) {
			sb.append(" (Alias: "); //$NON-NLS-1$
			sb.append(alias);
			sb.append(")="); //$NON-NLS-1$
		} else {
			sb.append("="); //$NON-NLS-1$
		}
		sb.append(getValue());
		
		return sb.toString();
	}

	public String getKeyLabel() {
		return defn.getKeyLabel();
	}

}
