package com.fundi.framework.common.navigation;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.util.ArrayList;
import java.util.List;

import com.fundi.framework.common.displaylist.DisplayList;
import com.fundi.framework.common.displaylist.ListFilter;
import com.fundi.framework.common.displaylist.ListFilterCondition;
import com.fundi.framework.common.helpers.ListFilterHelper;
import com.fundi.framework.common.instance.AppInstance;
import com.fundi.framework.common.instance.Environment;
import com.fundi.framework.common.nl1.Messages;

public class ToolMenuProvider extends NavigationProvider {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	public static final String 	ANY_FIELD = Messages.getString("ToolMenuProvider.0"); //$NON-NLS-1$
	private static final int	HISTORY_LIMIT = 20;	
	private enum FilterAction {
		MANAGE,
		SET_ACTIVE,
		SET_HIGHLIGHT, 
		SET_CASEINSENSITIVE,
		SET_HIGHLIGHTONLY
	};
	
	public ToolMenuProvider() {
	}

	@Override
	public void initialise(AppInstance instance) {
		super.initialise(instance);
	}

	/**
	 * Represents the filters for the given DisplayList as a menu heirarchy with links to open the Filter Management dialog
	 * 
	 *@param sourceObject	the DisplayList being filtered.
	 */
	@Override
	public ArrayList<MenuItem> getFilterMenuItems(Environment environment,	Object sourceObject) {
		ArrayList<MenuItem>		result;
		DisplayList				list;
		MenuItem				item,
								filterItem,
								conditionItem;
		ListFilterHelper		helper;
		
		result = new ArrayList<MenuItem>();
		helper = (ListFilterHelper) sourceObject;
		list = helper.getDisplayList();
		result.add(new MenuItem(Messages.getString("ToolMenuProvider.1"), Messages.getString("ToolMenuProvider.2"), Messages.getString("ToolMenuProvider.3"), null, AppInstance.Flag.DEFAULT));	//needs to contain at least one item to show up in the main menu properly //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		if (ListFilterHelper.class.isInstance(sourceObject)) {
			for (ListFilter filter: helper.retrieveSavedFilters()) {
				filterItem = new MenuItem(filter.getDescription(), Messages.getString("ToolMenuProvider.4"), Messages.getString("ToolMenuProvider.5") + filter.getDescription() + Messages.getString("ToolMenuProvider.6"), null, MenuItem.Style.CASCADE); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				filterItem.setData("filter", filter); //$NON-NLS-1$
				result.add(filterItem);
				
				item = new MenuItem(
						(filter.isActive() ? Messages.getString("ToolMenuProvider.8") : Messages.getString("ToolMenuProvider.9")),  //$NON-NLS-1$ //$NON-NLS-2$
						Messages.getString("ToolMenuProvider.10"),  //$NON-NLS-1$
						(filter.isActive() ? Messages.getString("ToolMenuProvider.11") : Messages.getString("ToolMenuProvider.12")),  //$NON-NLS-1$ //$NON-NLS-2$
						this, 
						filter.isActive(), 
						MenuItem.Style.PUSH
						);
				item.setData("action", FilterAction.SET_ACTIVE); //$NON-NLS-1$
				item.setData("filter", filter); //$NON-NLS-1$
				filterItem.addSubItem(item);

				item = new MenuItem(Messages.getString("ToolMenuProvider.15"), Messages.getString("ToolMenuProvider.16"), Messages.getString("ToolMenuProvider.17"), null, null); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				item.setData("action", FilterAction.MANAGE); //$NON-NLS-1$
				item.setData("filter", filter); //$NON-NLS-1$
				filterItem.addSubItem(item);
				for (ListFilterCondition condition: filter.getConditions()) {
					conditionItem = new MenuItem(condition.toString(), Messages.getString("ToolMenuProvider.20"), Messages.getString("ToolMenuProvider.21") + condition.toString(), null, null, MenuItem.Style.PUSH); //$NON-NLS-1$ //$NON-NLS-2$
					item.setData("action", FilterAction.MANAGE); //$NON-NLS-1$
					conditionItem.setData("filter", filter); //$NON-NLS-1$
					conditionItem.setData("condition", condition); //$NON-NLS-1$
					filterItem.addSubItem(conditionItem);
				}
			}
		}
		
		return result;
	}

	/**
	 * Every MenuItem returned from this list must include a reference to <b>this</b> to allow modification of the list by run(...)
	 * 
	 * @param sourceObject	a List&lt;String&gt; of search history.
	 */
	@SuppressWarnings("unchecked")
	@Override
	public ArrayList<MenuItem> getFindMenuItems(Environment environment, Object sourceObject) {
		ArrayList<MenuItem>			result;
		List<DisplayList.FindItem>	list;
		MenuItem					item;
		
		result = new ArrayList<MenuItem>();
		if (List.class.isInstance(sourceObject)) {
			list = (List<DisplayList.FindItem>) sourceObject;
			result.add(new MenuItem(Messages.getString("ToolMenuProvider.25"), Messages.getString("ToolMenuProvider.26"), Messages.getString("ToolMenuProvider.27"), this, AppInstance.Flag.DEFAULT, MenuItem.Style.PUSH)); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			for (DisplayList.FindItem findItem: list) {
				item = new MenuItem(findItem.toString(), Messages.getString("ToolMenuProvider.28"), Messages.getString("ToolMenuProvider.29") + findItem.toString() + Messages.getString("ToolMenuProvider.30"), this, findItem, MenuItem.Style.PUSH); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				result.add(item);
			}
		}
		
		return result;
	}

	@Override
	public void run(MenuItem item, Environment environment, Object sourceObject, Object sourceViewer) {
		if (ListFilterHelper.class.isInstance(sourceObject)) {
			runFilterItem(item, (ListFilterHelper) sourceObject);
		} else if (List.class.isInstance(sourceObject)) {
			runFindItem(item, (List) sourceObject);
		}
	}

	/**
	 * @param item	A MenuItem representing the selected filtering component and the action to take on it, likely to be a MenuItem constructed by getFilterMenuItems(...) above.
	 * @param list	The DisplayList to which the filtering is being applied.
	 */
	private void runFilterItem(MenuItem item, ListFilterHelper helper) {
		ListFilter				filter;
		ListFilterCondition		condition;
		boolean					save;

		filter = (ListFilter) item.getData("filter"); //$NON-NLS-1$
		condition = (ListFilterCondition) item.getData("condition"); //$NON-NLS-1$
		save = true;
		switch ((FilterAction) item.getData("action")) { //$NON-NLS-1$
		case SET_ACTIVE:
			filter.setActive(!((Boolean) item.getData()));
			break;

		case SET_CASEINSENSITIVE:
			condition.setCaseInsensitive(!((Boolean) item.getData()));
			break;

		case SET_HIGHLIGHT:
			condition.setHighlight(!((Boolean) item.getData()));
			break;
			
		default:
			save = false;
			break;
		}
		if (save) {
			helper.saveFilter(filter);
		}
	}

	/**
	 * @param item	A MenuItem representing the current Find or Locate operation, either taken from the list of history or if a new Find/Locate then
	 * 				a newly constructed MenuItem with the keyword/key in the data field.
	 * @param list	Expects a List&lt;String&gt; of keyword/key history (as passed to getFindMenuItems(...) before showing the menu) <i>to be updated</i>
	 */
	private void runFindItem(MenuItem item, List list) {
		Object	findItem;
		
		findItem = item.getData();
		list.remove(findItem);	//rely on implementation of findItem equals method
		list.add(0, findItem);	//rely on calling code to be consistent with list element type and item being added
		if (list.size() > HISTORY_LIMIT) {
			list.subList(HISTORY_LIMIT, list.size()).clear();
		}
	}
}
