package com.fundi.framework.common.util;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Allows the specification of an array of String search values and a 1-1 mapped array of String replace values. These will then be replaced in an input String
 * in an efficient single-pass manner. This implementation is significantly faster than running the same number of String.replace calls and intended to replace it.
 * 
 * Implementations should create a StringReplacer configured with the required search Strings and persist that instance for re-use in subsequent replace calls when
 * feasible.
 * 
 * The search values should be specified as a regex, as internally all search values will be concantenated into a regex string as "value1|value2|...|valueN"
 */
public class StringReplacer {
	
	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private List<String> 	searchValues;
	private List<String> 	replacementValues;
	private Pattern 		escapedPattern;
	
	public StringReplacer(String[] searches, String[] replacements) {
		StringBuilder patternString = new StringBuilder();
		
		searchValues = new ArrayList<String>(Arrays.asList(searches));
		replacementValues = new ArrayList<String>(Arrays.asList(replacements));
		for (String value: searchValues) {
			if (patternString.length() > 0) {
				patternString.append("|"); //$NON-NLS-1$
			}
			patternString.append(value);
		}
		escapedPattern = Pattern.compile(patternString.toString());
	}

	public String replace(String input) {
		StringBuffer result = new StringBuffer(input.length());
		Matcher	matcher = escapedPattern.matcher(input);
		String	match;
		
		try {
			while (matcher.find()) {
				match = matcher.group();
				matcher.appendReplacement(result, replacementValues.get(searchValues.indexOf(match)));
			}
			matcher.appendTail(result);
		} catch (ArrayIndexOutOfBoundsException aioobe) {
			aioobe.printStackTrace();
		}
		
		return result.toString();
	}
}
